@@ -35,6 +35,8 @@
 #include <ctype.h>
 #include <errno.h>
 
+#include <netinet/in.h>
+
 #include "http.h"
 #include "utils.h"
 
@@ -43,6 +45,7 @@
 */
 
 #define BUFSIZE                 1024
+#define BUFSIZE_STR		"1023"
 #define DEFAULT_HTTP_PORT         80
 
 typedef struct http_s http_t;
@@ -499,7 +502,7 @@ abort:
 	    this->contentlength = contentlength;
 	  }
         }
-        if (sscanf(this->buf, "Content-Type: %*s", sizeof (mime_type), mime_type) == 1)
+        if (sscanf(this->buf, "Content-Type: %"BUFSIZE_STR"s", mime_type) == 1)
         {
 	  logprintf ("http: content type = '%s'\n", mime_type);
 	  strcpy (this->mime_type, mime_type); /* note buffer sizes */
@@ -578,7 +581,7 @@ char *http_download (const char *url, ssize_t *file_size)
     }
   }
 
-  logprintf ("http: got %d bytes of content\n%s", total, buf);
+  logprintf ("http: got %zd bytes of content\n%s", total, buf);
 
   close (http->fh);
 
@@ -600,7 +603,7 @@ char *http_download (const char *url, ssize_t *file_size)
 ssize_t http_peek (const char *url, char *buffer, size_t buffer_size,
 	       char *mime_type, size_t mime_type_size)
 {
-  logprintf ("http: attempt to download %u bytes from %s\n", buffer_size, url);
+  logprintf ("http: attempt to download %zu bytes from %s\n", buffer_size, url);
 
   http_t *http = http_open (url);
   if (!http)
