@@ -300,6 +300,10 @@ JS_CB_T (Deinterlace)
 JS_CB_T (VideoPP) { gtk_video_set_use_post_plugins_video ((GtkVideo *)gtv, v); }
 JS_CB (DeinterlaceConf, "deinterlace_show ();")
 JS_CB (VideoPPConf, "postproc_video_show ();")
+JS_CB_T (WMUnblank)
+{
+  gtk_video_set_windowed_unblank ((GtkVideo *)gtv, v);
+}
 
 /* Audio */
 JS_CB (AudioSettings, "settings_show (1);")
@@ -357,16 +361,16 @@ static const GtkActionEntry main_actions[] = {
   { "SubsMenu",		NULL,			N_("_Subtitles"),		NULL },
   { "VideoMenu",	NULL,			N_("V_ideo"),			NULL },
   CB(VideoSettings,	GTK_STOCK_PROPERTIES,	/* Video */ N_("_Settings..."),	NULL),
-  { "VideoConfigMenu",	NULL,			N_("_Configure"),		NULL },
-  CB(DeinterlaceConf,	GTK_STOCK_PREFERENCES,	N_("_Deinterlace..."),		NULL),
-  CB(VideoPPConf,	GTK_STOCK_PREFERENCES,	N_("_Post-processing..."), 	NULL),
+  { "VideoConfigMenu",	GTK_STOCK_PREFERENCES,	N_("_Configure"),		NULL },
+  CB(DeinterlaceConf,	NULL,			N_("_Deinterlace..."),		NULL),
+  CB(VideoPPConf,	NULL,			N_("_Video post-processing..."),NULL),
   { "AudioMenu",	NULL,			N_("_Audio"),			NULL },
   CB(AudioSettings,	GTK_STOCK_PROPERTIES,	/* Audio */ N_("_Settings..."),	NULL),
   CB(AudioEq,		GTK_STOCK_PROPERTIES,	N_("_Equaliser..."),		NULL),
-  { "AudioConfigMenu",	NULL,			N_("_Configure"),		NULL },
-  CB(AudioPPConf,	GTK_STOCK_PREFERENCES,	N_("_Post-processing..."),	NULL),
+  { "AudioConfigMenu",	GTK_STOCK_PREFERENCES,	N_("_Configure"),		NULL },
+  CB(AudioPPConf,	NULL,			N_("_Audio post-processing..."),NULL),
   { "MediaMenu",	NULL,			N_("_Media"),			NULL },
-  CB(MediaAdd,		GTK_STOCK_NEW,		N_("_Add media mark.."),	"<control>D"),
+  CB(MediaAdd,		GTK_STOCK_NEW,		N_("_Add media mark..."),	"<control>D"),
   CB(MediaManage,	GXINE_MEDIA_MARK,	N_("_Manage media marks..."),	"<control>B"),
   CB(MediaImport,	GTK_STOCK_OPEN,		N_("_Import media marks..."),	NULL),
   { "ToolsMenu",	NULL,			N_("_Tools"),			NULL },
@@ -391,6 +395,7 @@ typedef enum {
   TOGGLE_DEINTERLACE,
   TOGGLE_VIDEO_PP,
   TOGGLE_AUDIO_PP,
+  TOGGLE_UNBLANK,
 } toggle_actions_e;
 static GtkToggleActionEntry toggle_actions[] = {
   CB(Fullscreen,	NULL, N_("_Full-screen mode"),		"<control>F",	FALSE),
@@ -402,11 +407,13 @@ static GtkToggleActionEntry toggle_actions[] = {
   CB(Deinterlace,	NULL, N_("_Deinterlace"),		"<control>I",	FALSE),
   CB(VideoPP,		NULL, N_("_Post-processing"),		NULL,		FALSE),
   CB(AudioPP,		NULL, N_("_Post-processing"),		NULL,		FALSE),
+  CB(WMUnblank,		NULL, N_("Prevent _blanking\nin windowed mode"), NULL,	FALSE),
 };
 static const char *const toggle_prefs[] = {
   NULL, NULL, NULL, NULL, NULL,
   "gui.magnify_lowres_video", "gui.post_plugins.deinterlace_enable",
   "gui.post_plugins.video_enable", "gui.post_plugins.audio_enable",
+  "gui.windowedmode_unblank"
 };
 #undef CB
 
@@ -470,7 +477,13 @@ static GtkWidget *add_menu_ui (GtkUIManager *ui, int i)
 	   "<menuitem action='Prefs' />\n"
 	   "<menuitem action='Keys' />\n"
 	   "<menuitem action='Startup' />\n"
+	   "<separator />\n"
+	   /* the next 3 items duplicate items in the video & audio menus */
+	   "<menuitem action='DeinterlaceConf' />\n"
+	   "<menuitem action='VideoPPConf' />\n"
+	   "<menuitem action='AudioPPConf' />\n"
 #ifdef HAVE_LIRC
+	   "<separator />\n"
 	   "<menuitem action='ReloadLIRC' />\n"
 #endif
 	   "<separator />\n"
@@ -555,6 +568,7 @@ static GtkWidget *add_menu_ui (GtkUIManager *ui, int i)
 	    "<menuitem action='DeinterlaceConf' />\n"
 	    "<menuitem action='VideoPPConf' />\n"
 	  "</menu>\n"
+	  "<menuitem action='WMUnblank' />\n"
 	"</menu>\n"
 	"<menu action='AudioMenu'>\n"
 	  "<menuitem action='AudioSettings' />\n"
@@ -608,7 +622,9 @@ GtkAction *find_video_size_action (gdouble factor)
 
 void scale_changed_cb (GtkWidget *widget, gdouble factor, gpointer data)
 {
-  if (factor != -1 && action_items.resize_factor)
+  if (factor != -1 && action_items.resize_factor
+      && xine_get_status (stream) != XINE_STATUS_IDLE
+      && GTK_WIDGET_VISIBLE (gtv))
   {
     logprintf ("gxine: received scale change event (%lf%%)\n", factor);
     GtkAction *action = find_video_size_action (factor);
@@ -702,6 +718,7 @@ void create_menus (GtkWidget *window)
   action_items.fs_toolbar = get_toggle ("FSToolbarVisible");
   action_items.wm_toolbar = get_toggle ("WMToolbarVisible");
   action_items.wm_toolbar_snap = get_toggle ("WMToolbarSnap");
+  action_items.wm_unblank = get_toggle ("WMUnblank");
 
   action_items.aspect = get_radio_group ("AspectAuto");
   action_items.vis = get_radio_group ("VisNone");
