@@ -66,7 +66,7 @@ static GtkListStore   *pl_store;
 static GtkWidget      *dlg;
 static GtkTreeView    *tree_view;
 static GtkTreeSelection *sel;
-static GtkWidget      *repeat_button, *random_button;
+static GtkToggleButton *repeat_button, *random_button;
 static char            logo_mrl[1024];
 static int             logo_mode;
 
@@ -290,7 +290,7 @@ static void paste_cb (GtkWidget* widget, gpointer data)
 static void new_cb (GtkWidget* widget, gpointer data)
 {
   play_item_t *play_item = play_item_new ("", "", 0);
-  if (play_item_edit (play_item, PLAY_ITEM_LIST, NULL, dlg))
+  if (play_item_edit (play_item, PLAY_ITEM_LIST_NEW, NULL, dlg))
     playlist_add (play_item, -1);
   else
     play_item_dispose (play_item);
@@ -448,11 +448,9 @@ static int playlist_load (const char *fname)
       {
         /* gxine-specific */
 	gtk_toggle_button_set_active
-	  (GTK_TOGGLE_BUTTON (random_button),
-	   xml_parser_get_property_bool (node, "RANDOM", 0));
+	  (random_button, xml_parser_get_property_bool (node, "RANDOM", 0));
 	gtk_toggle_button_set_active
-	  (GTK_TOGGLE_BUTTON (repeat_button),
-	   xml_parser_get_property_bool (node, "REPEAT", 0));
+	  (repeat_button, xml_parser_get_property_bool (node, "REPEAT", 0));
       }
 
       else if (!strcasecmp (node->name, "ENTRY"))
@@ -528,8 +526,8 @@ void playlist_save (char *file)
     fprintf
       (f, "<ASX VERSION=\"3.0\">\n"
           "  <SETTINGS REPEAT=\"%s\" RANDOM=\"%s\"/>\n",
-       tf[!!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(repeat_button))],
-       tf[!!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(random_button))]);
+       tf[!!gtk_toggle_button_get_active (repeat_button)],
+       tf[!!gtk_toggle_button_get_active (random_button)]);
 
     if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (pl_store), &iter))
     {
@@ -1518,8 +1516,8 @@ static void play_next (void)
 {
   int is_repeat, is_random;
 
-  is_repeat = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (repeat_button));
-  is_random = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (random_button));
+  is_repeat = gtk_toggle_button_get_active (repeat_button);
+  is_random = gtk_toggle_button_get_active (random_button);
 
   unmark_play_item ();
   ref_list_pos = -1;
@@ -1586,7 +1584,9 @@ static void xine_event_cb (void *user_data, const xine_event_t *event)
     ui_set_control_adjustment (Control_SEEKER, 0);
     if (!logo_mode)
     {
+      gdk_threads_leave();
       se_eval (gse, "eval (event.stream_end);", NULL, NULL, NULL, "event.stream_end");
+      gdk_threads_enter();
       play_next ();
     }
     gdk_threads_leave();
@@ -1714,7 +1714,7 @@ typedef struct {
       typeof (display_info) *msg;
       const char *fallback = NULL;
 
-      /*gdk_threads_enter();*/
+      gdk_threads_enter();
 
       switch (data->type)
       {
@@ -1769,7 +1769,8 @@ typedef struct {
 	msg (FROM_XINE, fallback, "");
       else
 	msg (FROM_XINE, NULL, _("Message code %d"), data->type);
-      /*gdk_threads_leave();*/
+
+      gdk_threads_leave();
     }
     break;
 
@@ -1865,24 +1866,24 @@ static void drop_cb (GtkTreeView	*widget,
     if (ins_pos == -1)
     {
       gtk_list_store_move_before (pl_store, &from, NULL);
-      ins_pos = gtk_tree_model_iter_n_children (drag_mrl.model, NULL);
+      ins_pos = gtk_tree_model_iter_n_children (drag_mrl.model, NULL) - 1;
     }
     else
     {
       gtk_tree_model_iter_nth_child (drag_mrl.model, &to, NULL, ins_pos);
+      if (ins_pos >= drag_mrl.index)
+	--ins_pos;
       gtk_list_store_move_before (pl_store, &from, &to);
     }
 
     /* adjust index of currently-being-played item if necessary */
-    if (cur_list_pos != -1)
+    if (cur_list_pos >= 0)
     {
       if (cur_list_pos == drag_mrl.index)
 	cur_list_pos = ins_pos;
-      if (drag_mrl.index < ins_pos && drag_mrl.index <= cur_list_pos
-	  && cur_list_pos <= ins_pos)
+      else if (drag_mrl.index < cur_list_pos && cur_list_pos <= ins_pos)
 	--cur_list_pos;
-      if (drag_mrl.index > ins_pos && drag_mrl.index >= cur_list_pos
-	  && cur_list_pos >= ins_pos)
+      else if (drag_mrl.index > cur_list_pos && cur_list_pos >= ins_pos)
 	++cur_list_pos;
     }
 
@@ -2164,6 +2165,26 @@ static void check_list_empty (void)
   ui_set_status (UI_CURRENT_STATE);
 }
 
+static void get_playlist_repeat (se_prop_t *prop, se_prop_read_t *value)
+{
+  value->i = gtk_toggle_button_get_active (repeat_button);
+}
+
+static void get_playlist_random (se_prop_t *prop, se_prop_read_t *value)
+{
+  value->i = gtk_toggle_button_get_active (random_button);
+}
+
+static void set_playlist_repeat (int v)
+{
+  gtk_toggle_button_set_active (repeat_button, v);
+}
+
+static void set_playlist_random (int v)
+{
+  gtk_toggle_button_set_active (random_button, v);
+}
+
 void playlist_init (void)
 {
   GtkWidget *hbox, *scrolled_window;
@@ -2280,13 +2301,13 @@ void playlist_init (void)
   hbox = gtk_hbox_new (0,2);
   gtk_box_pack_start (vbox, hbox, FALSE, FALSE, 2);
 
-  repeat_button = gtk_check_button_new_with_mnemonic (_("_Repeat"));
+  repeat_button = (GtkToggleButton *)gtk_check_button_new_with_mnemonic (_("_Repeat"));
   gtk_box_pack_start (GTK_BOX (hbox), repeat_button, FALSE, FALSE, 2);
 
-  random_button = gtk_check_button_new_with_mnemonic (_("_Random"));
+  random_button = (GtkToggleButton *)gtk_check_button_new_with_mnemonic (_("_Random"));
   gtk_box_pack_start (GTK_BOX (hbox), random_button, FALSE, FALSE, 2);
 
-  sprintf (logo_mrl, "%s/logo.mpv", logodir);
+  sprintf (logo_mrl, "%s/logo."LOGO_FORMAT, logodir);
   logo_mode = 1;
 
   playlist_load (NULL);
@@ -2320,6 +2341,14 @@ void playlist_init (void)
 	SE_GROUP_HIDDEN, NULL, NULL },
       { NULL }
     };
+    static const ui_property_t props[] = {
+      { "repeat", N_("v=bool, toggle()"), 0, get_playlist_repeat, set_playlist_repeat },
+      { "random", N_("v=bool, toggle()"), 0, get_playlist_random, set_playlist_random },
+      { NULL }
+    };
+    se_o_t *playlist = se_create_object (gse, NULL, "playlist", NULL,
+		       SE_GROUP_PROPERTIES, NULL);
+    ui_create_properties (props, playlist, SE_TYPE_BOOL);
     se_defuns (gse, NULL, defs);
   }
 }
