@@ -187,7 +187,14 @@ void server_setup (void)
 {
   char filename[FILENAME_MAX];
   snprintf (filename, sizeof (filename), SOCKET_FILENAME, getenv ("HOME"));
-  gxsocket = make_socket (filename);
+  if (strlen (filename) >= sizeof (((struct sockaddr_un *)0)->sun_path))
+  {
+    fputs (_("server: name too long - gxine_client will not be able to connect\n"),
+	   stderr);
+    gxsocket = -1;
+  }
+  else
+    gxsocket = make_socket (filename);
 }
 
 void server_start (void)
@@ -238,12 +245,14 @@ int server_client_connect (void)
     struct sockaddr s;
   } cli_adr = { .u = {0} }, serv_adr = { .u = {0} };
   int                length = sizeof(struct sockaddr_un);
-  char               filename[PATH_MAX];
+  char               filename[FILENAME_MAX];
 
   signal (SIGPIPE, sigpipe_handler);
 
   /* server filename */
-  snprintf (filename, PATH_MAX, SOCKET_FILENAME, getenv ("HOME"));
+  snprintf (filename, sizeof (filename), SOCKET_FILENAME, getenv ("HOME"));
+  if (strlen (filename) >= sizeof (cli_adr.u.sun_path))
+    return 0; /* no point in printing anything... */
 
   logprintf ("server: trying to connect to already running instance of gxine (%s)...\n",
 	    filename);
